#include <mainwin.h>

/******************* CONTRIBUTED BY KUMAR RAHUL RANJAN *********************
***************************************************************************/    
/* In ACI ,the 8-bit data (operand) and the Carry flag are added to the
contents of the accumulator and the result is stored in the accumulator.
All flags are modified to reflect the result of the addition.*/
void MainWin::ACI(int n)
{
    Register[IndexOfRegister["A"]] += n + Flag[IndexOfFlag["CY"]];
    ChangedRegs.push_back("A");
    UpdateFlags("A");
}

/* IN ADC the contents of the operand (register or memory) and the Carry
flag are added to the contents of the accumulator and the result is stored
in the accumulator. If the operand is a memory location, its location is
specified by the contents of the HL registers. All flags are modified to
reflect the result of the addition.*/
void MainWin::ADC(std::string a)
{
    if(a != "M")
        Register[IndexOfRegister["A"]] += Register[IndexOfRegister[a]] + Flag[IndexOfFlag["CY"]];
    else
        Register[IndexOfRegister["A"]] += Memory[Register[IndexOfRegister["HL"]]] + Flag[IndexOfFlag["CY"]];
    UpdateFlags("A");
    ChangedRegs.push_back("A");
}

/* In ADD the contents of the operand (register or memory) are added to the
contents of the accumulator and the result is stored in the accumulator. If
the operand is a memory location, its location is specified by the contents
of the HL registers. All flags are modified to reflect the result of the addition.*/
void MainWin::ADD(std::string a)
{
    if(a != "M")
        Register[IndexOfRegister["A"]] += Register[IndexOfRegister[a]];
    else
        Register[IndexOfRegister["A"]] += Memory[Register[IndexOfRegister["HL"]]];
    UpdateFlags("A");
    ChangedRegs.push_back("A");
}

/*In ADI the 8-bit data (operand) is added to the contents of the accumulator
and the result is stored in the accumulator. All flags are modified to reflect
the result of the addition.*/
void MainWin::ADI(int n)
{
    Register[IndexOfRegister["A"]] += n;
    UpdateFlags("A");
    ChangedRegs.push_back("A");
}

/* In ANA the 8-bit data (operand) is added to the contents of the accumulator
and the result is stored in the accumulator. All flags are modified to reflect
the result of the addition.*/
void MainWin::ANA(std::string a)
{
    if(a != "M")
        Register[IndexOfRegister["A"]] &= Register[IndexOfRegister[a]];
    else
        Register[IndexOfRegister["A"]] &= Memory[Register[IndexOfRegister["HL"]]];
    UpdateFlags("A");
    ChangedRegs.push_back("A");
    Flag[IndexOfFlag["CY"]] = 0;
    Flag[IndexOfFlag["AC"]] = 1;
    UpdateAF();
}

/********************** CONTRIBUTED BY SASWAT PADHI *************************
*****************************************************************************/    
/*
    void MainWin::POP(string register_name)

    Expected Argument : B, D, H, PSW

    POPs the top-most value out of the program stack and,
    stores it in register_name register.

    Affects flag : SP
    Affects register : <register_name>
*/
void MainWin::POP(std::string register_name)
{
    if(register_name == "PSW")
    {
        Register[IndexOfRegister["SP"]]++;
        Register[IndexOfRegister["A"]] = Stack[Register[IndexOfRegister["SP"]]];
        Register[IndexOfRegister["SP"]]++;
        Register[IndexOfRegister["F"]] = Stack[Register[IndexOfRegister["SP"]]];
        ChangedRegs.push_back("A");
        UpdateAF();
    }
    else
    {
        Register[IndexOfRegister["SP"]]++;
        Register[IndexOfRegister[register_name]+1] = Stack[Register[IndexOfRegister["SP"]]];
        Register[IndexOfRegister["SP"]]++;
        Register[IndexOfRegister[register_name]] = Stack[Register[IndexOfRegister["SP"]]];

        ChangedRegs.push_back(register_name);
        if(register_name == "B")    ChangedRegs.push_back("C");
        else if(register_name == "D")    ChangedRegs.push_back("E");
        else if(register_name == "H")    ChangedRegs.push_back("L");
    }

    UpdateBC();
    UpdateDE();
    UpdateHL();
}

/*
    void MainWin::XTHL()

    Expected Argument : <none>

    Exchanges the top-most stack value with HL register

    Affects flag : <none>
    Affects register : <none>
*/
void MainWin::XTHL()
{
    int Temp;
    Temp = Register[IndexOfRegister["L"]];
    Register[IndexOfRegister["L"]] = Stack[Register[IndexOfRegister["SP"]]];
    Stack[Register[IndexOfRegister["SP"]]] = Temp;

    Temp = Register[IndexOfRegister["H"]];
    Register[IndexOfRegister["H"]] = Stack[Register[IndexOfRegister["SP"]]+1];
    Stack[Register[IndexOfRegister["SP"]]+1] = Temp;
    
    ChangedRegs.push_back("H");
    ChangedRegs.push_back("L");
    UpdateHL();
}

/*
    void MainWin::CALL(int address)

    Expected Argument : Memory address in valid range

    Calls a sub-routine at the specified address after storing
    the current address in the stack.

    Affects flag : SP
    Affects register : PC
*/
void MainWin::CALL(int address)
{
    Register[IndexOfRegister["SP"]]--;
    Stack[Register[IndexOfRegister["SP"]]] = Register[IndexOfRegister["PC"]];
    Register[IndexOfRegister["PC"]] = address;
}

/*
    void MainWin::CM(int address)

    Expected Argument : Memory address in valid range

    Conditionally calls a sub-routine at the specified address through
    CALL function, if Sign flag is SET i.e. negative.

    Affects flag : SP
    Affects register : PC
*/
void MainWin::CM(int address)
{
    if(Flag[IndexOfFlag["S"]])
        CALL(address);
}

/*
    void MainWin::CM(int address)

    Expected Argument : Memory address in valid range

    Conditionally calls a sub-routine at the specified address through
    CALL function, if Carry flag is SET.

    Affects flag : SP
    Affects register : PC
*/
void MainWin::CC(int address)
{
    if(Flag[IndexOfFlag["CY"]])
        CALL(address);
}

/*
    void MainWin::RET()

    Expected Argument : <none>

    RETurns control from a sub-routine which was CALLed.
    POPs the stored address from stack.

    Affects flag : SP
    Affects register : PC
*/
void MainWin::RET()
{
    Register[IndexOfRegister["PC"]] = Stack[Register[IndexOfRegister["SP"]]] + 1;
    Register[IndexOfRegister["SP"]]++;
}

/*
    void MainWin::RNC()

    Expected Argument : <none>

    Conditionally returns control from a sub-routine through
    a RET call if Carry flag is not SET.

    Affects flag : SP
    Affects register : PC
*/
void MainWin::RNC()
{
    if(!Flag[IndexOfFlag["CY"]])
        RET();
}

/*
    void MainWin::RNZ()

    Expected Argument : <none>

    Conditionally returns control from a sub-routine through
    a RET call if Zero flag is not SET.

    Affects flag : SP
    Affects register : PC
*/
void MainWin::RNZ()
{
    if(!Flag[IndexOfFlag["Z"]])
        RET();
}


/**********************  CONTRIBUTED BY RAGHU TEJA  *************************
*****************************************************************************/    
void MainWin::MOV (std::string a, std::string b)
{

    if(a == "M")
        Memory[Register[IndexOfRegister["HL"]]] = Register[IndexOfRegister[b]];
    else if(b == "M")
        Register[IndexOfRegister[a]] = Memory[Register[IndexOfRegister["HL"]]];
    else
        Register[IndexOfRegister[a]]=Register[IndexOfRegister[b]];

    if(a != "M")
        ChangedRegs.push_back(a);

    UpdateAF();
    UpdateBC();
    UpdateDE();
    UpdateHL();
}

void MainWin::MVI (std::string a, int n)
{
    if(a == "M")
        Memory[Register[IndexOfRegister["HL"]]] = n;
    else
        Register[IndexOfRegister[a]] = n;

    if(a != "M")
        ChangedRegs.push_back(a);

    UpdateAF();
    UpdateBC();
    UpdateDE();
    UpdateHL();
}      

void MainWin::NOP ()
{
    ;
}

void MainWin::ORA(std::string a)
{
    if(a != "M")
        Register[IndexOfRegister["A"]] |= Register[IndexOfRegister[a]];
    else
        Register[IndexOfRegister["A"]] |= Memory[Register[IndexOfRegister["HL"]]];

    UpdateFlags("A");
    ChangedRegs.push_back("A");

    Flag[IndexOfFlag["CY"]] = 0;
    Flag[IndexOfFlag["AC"]] = 1;
    UpdateAF();
}

void MainWin::ORI(int n)
{
    Register[IndexOfRegister["A"]] |= n;
    UpdateFlags("A");
    ChangedRegs.push_back("A");
    Flag[IndexOfFlag["CY"]] = 0;
    Flag[IndexOfFlag["AC"]] = 0;
    UpdateAF();
}


/********************** CONTRIBUTED BY NILESH JAGNIK *************************
*****************************************************************************/    
// The contents of the accumulator are complemented. No flags are affected.
void MainWin::CMA()
{
    Register[IndexOfRegister["A"]] = ~Register[IndexOfRegister["A"]];
    ChangedRegs.push_back("A");
    UpdateAF();
}

//The Carry flag is complemented. No other flags are affected.
void MainWin::CMC()
{
    Flag[IndexOfFlag["CY"]] = ~Flag[IndexOfFlag["CY"]];
    UpdateAF();
}

/*
The contents of the operand (register or memory) are
compared with the contents of the accumulator. Both
contents are preserved . The result of the comparison is
shown by setting the flags of the PSW as follows:
if (A) < (reg): carry flag is set
if (A) = (reg): zero flag is set
if (A) > (reg): carry and zero flags are reset
*/
void MainWin::CMP(std::string a)
{
    if(Register[IndexOfRegister["A"]] < Register[IndexOfRegister[a]])
        Flag[IndexOfFlag["CY"]]=1;
    else if(Register[IndexOfRegister["A"]] == Register[IndexOfRegister[a]])
        Flag[IndexOfFlag["Z"]]=1;
    else
    {
        Flag[IndexOfFlag["CY"]]=0;
        Flag[IndexOfFlag["Z"]]=0;
    }
    UpdateAF();
}

/*The program sequence is transferred to the memory location
specified by the 16-bit address given in the operand  if carry flag is set to 0   */
void MainWin::CNC(int a)
{
    if (Flag[IndexOfFlag["CY"]] == 0)
        CALL(a);
}

/*The program sequence is transferred to the memory location
specified by the 16-bit address given in the operand  if zero flag is set to 0   */
void MainWin::CNZ(int a)
{
    if (Flag[IndexOfFlag["Z"]] == 0)
        CALL(a);
}

/*The program sequence is transferred to the memory location
specified by the 16-bit address given in the operand  if sign flag is set to 0   */
void MainWin::CP(int a)
{
    if (Flag[IndexOfFlag["S"]]==0)
        CALL(a);
}

/*The program sequence is transferred to the memory location
specified by the 16-bit address given in the operand  if parity flag is set to 1   */
void MainWin::CPE(int a)
{
    if (Flag[IndexOfFlag["P"]]==1)
        CALL(a);
}

/*
The second byte (8-bit data) is compared with the contents of
the accumulator. The values being compared remain
unchanged. The result of the comparison is shown by setting
the flags of the PSW as follows:
if (A) < data: carry flag is set
if (A) = data: zero flag is set
if (A) > data: carry and zero flags are reset
*/
void MainWin::CPI(int n)
{
    if(Register[IndexOfRegister["A"]] < n)
        Flag[IndexOfFlag["CY"]]=1;
    else if(Register[IndexOfRegister["A"]] == n)
        Flag[IndexOfFlag["Z"]]=1;
    else
    {
        Flag[IndexOfFlag["CY"]]=0;
        Flag[IndexOfFlag["Z"]]=0;
    }
    UpdateAF();
}

/*The program sequence is transferred to the memory location
specified by the 16-bit address given in the operand  if parity flag is set to 0*/
void MainWin::CPO(int a)
{
    if (Flag[IndexOfFlag["P"]]==0)
        CALL(a);
}

/*The program sequence is transferred to the memory location
specified by the 16-bit address given in the operand  if zero flag is set to 1*/
void MainWin::CZ(int a)
{
    if (Flag[IndexOfFlag["Z"]]==1)
        CALL(a);
}


/******************* CONTRIBUTED BY RAGHU TEJA [AGAIN] **********************
*****************************************************************************/    
void MainWin::XCHG ()
{
	char temp = Register[IndexOfRegister["HL"]];
	Register[IndexOfRegister["HL"]]=Register[IndexOfRegister["DE"]];
	Register[IndexOfRegister["DE"]]=temp;

        ChangedRegs.push_back("H");
        ChangedRegs.push_back("L");
        ChangedRegs.push_back("D");
        ChangedRegs.push_back("E");

	UpdateH_L();
	UpdateD_E();
}

void MainWin::XRA(std::string a)
{
    if(a != "M")
        Register[IndexOfRegister["A"]] ^= Register[IndexOfRegister[a]];
    else
        Register[IndexOfRegister["A"]] ^= Memory[Register[IndexOfRegister["HL"]]];
    UpdateFlags("A");
    ChangedRegs.push_back("A");
    Flag[IndexOfFlag["CY"]] = 0;
    Flag[IndexOfFlag["AC"]] = 1;
    UpdateAF();
}

void MainWin::XRI(int n)
{
    Register[IndexOfRegister["A"]] ^= n;
    UpdateFlags("A");
    ChangedRegs.push_back("A");
    Flag[IndexOfFlag["CY"]] = 0;
    Flag[IndexOfFlag["AC"]] = 0;
    UpdateAF();
}

void MainWin::DCX(std::string a)
{
    if(a == "B")
    {
        a = "BC";
        ChangedRegs.push_back("B");
        ChangedRegs.push_back("C");
        UpdateB_C();
    }
    else if(a == "D")
    {
        a = "DE";
        ChangedRegs.push_back("D");
        ChangedRegs.push_back("E");
        UpdateD_E();
    }
    else if(a == "H" || a == "M")
    {
        a = "HL";
        ChangedRegs.push_back("H");
        ChangedRegs.push_back("L");
        UpdateH_L();
    }
    else
    {
        ChangedRegs.push_back(a);
        Register[IndexOfRegister[a]]--;
    }
}

/*********************** CONTRIBUTED BY PULKIT BANSAL ***********************
*****************************************************************************/
void MainWin::JC (int address)
{
	if(Flag[IndexOfFlag["CY"]] == 1)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}

void MainWin::JM (int address)
{
	if(Flag[IndexOfFlag["S"]] == 1)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}

void MainWin::JNC (int address)
{
	if(Flag[IndexOfFlag["CY"]] == 0)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}

void MainWin::JNZ (int address)
{
	if(Flag[IndexOfFlag["Z"]] == 0)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}
void MainWin::JP (int address)
{
	if(Flag[IndexOfFlag["S"]] == 0)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}
void MainWin::JPE (int address)
{
	if(Flag[IndexOfFlag["P"]] == 1)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}
void MainWin::JPO (int address)
{
	if(Flag[IndexOfFlag["P"]] == 1)
    	Register[IndexOfRegister["PC"]] = Memory[address];
}


/*********************** CONTRIBUTED BY DAYA SHANKAR ***********************
*****************************************************************************/
// function for Load Accumulator direct
void MainWin::LDA (int a)
{
    Register[IndexOfRegister["A"]] = Memory[a];
    ChangedRegs.push_back("A");
    UpdateAF();
}

// function for Load Accumulator indirect
void MainWin::LDAX (std::string a)
{
    if(a == "B")
        Register[IndexOfRegister["A"]] = Memory[Register[IndexOfRegister["BC"]]];
    else if(a == "D")
        Register[IndexOfRegister["A"]] = Memory[Register[IndexOfRegister["DE"]]];
    ChangedRegs.push_back("A");
    UpdateAF();
}

// function of Load HL Direct
void MainWin::LHLD (int a)
{
    Register[IndexOfRegister["L"]] = Memory[a];
    Register[IndexOfRegister["H"]] = Memory[a+1];
    ChangedRegs.push_back("L");
    ChangedRegs.push_back("H");
    UpdateHL();
}

void MainWin::LXI (std::string a, int nn)
{
    if(a == "B")
    {
        Register[IndexOfRegister["BC"]]=nn;
        ChangedRegs.push_back("B");
        ChangedRegs.push_back("C");
        UpdateB_C();
    }
    if(a == "D")
    {
        Register[IndexOfRegister["DE"]]=nn;
        ChangedRegs.push_back("D");
        ChangedRegs.push_back("E");
        UpdateD_E();
    }
    if(a == "H")
    {
        Register[IndexOfRegister["HL"]]=nn;
        ChangedRegs.push_back("L");
        ChangedRegs.push_back("H");
        UpdateH_L();
    }
}


/*********************** CONTRIBUTED BY PAWAN NAGWANI ***********************
*****************************************************************************/
void MainWin::DAD(std::string r)
{
    if(r == "B")
            Register[IndexOfRegister["HL"]] += Register[IndexOfRegister["BC"]];
    else if(r== "D")
            Register[IndexOfRegister["HL"]] += Register[IndexOfRegister["DE"]];
    else if(r== "H")
            Register[IndexOfRegister["HL"]] += Register[IndexOfRegister["DE"]];
    ChangedRegs.push_back("H");
    ChangedRegs.push_back("L");
}

void MainWin::DCR(std::string r)
{
    if(r == "M")
            Memory[Register[IndexOfRegister["HL"]]]--;
    else
            Register[IndexOfRegister[r]]--;
    ChangedRegs.push_back(r);
    UpdateAF();
    UpdateBC();
    UpdateDE();
}


/******************* CONTRIBUTED BY YOGENDRA SINGH DABI *********************
*****************************************************************************/
/*The contents of the operand (register or memory ) are
subtracted from the contents of the accumulator, and the result
is stored in the accumulator. If the operand is a memory
location, its location is specified by the contents of the HL
registers. All flags are modified to reflect the result of the
subtraction.
Example: SUB B or SUB M*/
void MainWin::SUB(std::string r)
{
  Register[IndexOfRegister["A"]] -= Register[IndexOfRegister[r]];
  ChangedRegs.push_back("A");
  UpdateFlags("A");
}

/*The 8-bit data (operand) is subtracted from the contents of the
accumulator and the result is stored in the accumulator. All
flags are modified to reflect the result of the subtraction.
Example: SUI 45H*/
void MainWin::SUI(int n)
{
    Register[IndexOfRegister["A"]] -= n;
    ChangedRegs.push_back("A");
    UpdateFlags("A");
}

/*The contents of the operand (register )
the Borrow flag are subtracted from the contents of the
accumulator and the result is placed in the accumulator. If
the operand is a memory location, its location is specified by
the contents of the HL registers. All flags are modified to
reflect the result of the subtraction.
Example: SBB B */
void MainWin::SBB(std::string r)
{
    Register[IndexOfRegister["A"]] -= (Register[IndexOfRegister[r]] + Flag[IndexOfFlag["CY"]]);
    ChangedRegs.push_back("A");
    UpdateFlags("A");
}

/*The 8-bit data (operand) and the Borrow flag are subtracted
from the contents of the accumulator and the result is stored
in the accumulator. All flags are modified to reflect the result
of the subtracion.
Example: SBI 45H */
void MainWin::SBI(int n)
{
    Register[IndexOfRegister["A"]] -= (n + Flag[IndexOfFlag["CY"]]);
    ChangedRegs.push_back("A");
    UpdateFlags("A");
}

void MainWin::RAL()
{
    int msb = (Register[IndexOfRegister["A"]] > 127) ? 1 : 0; // msb stores the 8th bit(from right) of A register
    Register[IndexOfRegister["A"]] <<= 1;// performing bitwise shifting
    ChangedRegs.push_back("A");
    Flag[IndexOfFlag["CY"]]=msb;// 8th bit is stored in the carry flag
    UpdateAF();
}

void MainWin::RAR()
{
    int msb = (Register[IndexOfRegister["A"]] & 1);//msb stores the 1st bit (from right)
    Register[IndexOfRegister["A"]] >>= 1;//bitwise shifting
    ChangedRegs.push_back("A");
    Flag[IndexOfFlag["CY"]] = msb;
    UpdateAF();
}


/************************ CONTRIBUTED BY VIPUL SINGH ************************
*****************************************************************************/
void MainWin::PUSH (std::string a)                                                                        // pushing into stack
{
	if (a == "PSW")                                                                     //checking whether parameter equals Program Status Word
	{
		Stack[Register[IndexOfRegister["SP"]]] = Register[IndexOfRegister["F"]];  //Value of PSW put into stack top 
		Register[IndexOfRegister["SP"]]--;                                          //Stack Pointer again decremented
		Stack[Register[IndexOfRegister["SP"]]] = Register[IndexOfRegister["A"]];    //Value in register A put into new stack top
                Register[IndexOfRegister["SP"]]--;                                          //Stack Pointer again decremented
	}
	else
	{
		Stack[Register[IndexOfRegister["SP"]]] = Register[IndexOfRegister[a]];       //value of given register pushed                       
		Register[IndexOfRegister["SP"]]--;                                           //Stack Pointer again decremented
                Stack[Register[IndexOfRegister["SP"]]] = Register[IndexOfRegister[a]+1];     //value of next register pushed
                Register[IndexOfRegister["SP"]]--;                                          //Stack Pointer again decremented
	}
}

void MainWin::ANI(int n)
{
    Register[IndexOfRegister["A"]] &= n;
    ChangedRegs.push_back("A");
    UpdateFlags("A");
    Flag[IndexOfFlag["CY"]] = 0;
    Flag[IndexOfFlag["AC"]] = 0;
    UpdateAF();
}

void MainWin::RZ ()
{
    if ( Flag[IndexOfFlag["Z"]] == 1 )                                                  //Conditionally returns control from a sub-routine through
        RET ();                                                                             //a RET call if zero flag is currently 1
}

void MainWin::RP ()
{
    if ( Flag[IndexOfFlag["S"]] == 0 )     //Conditionally returns control from a sub-routine through a RET call if
        RET ();                                                                            //sign flag is currently 0.
}

void MainWin::RPO ()
{
	if ( Flag[IndexOfFlag["P"]] == 0 )						   //Conditionally returns control from a sub-routine through
        RET ();                                                                            //a RET call if parity flag is currently 0
}

void MainWin::RPE ()
{
	if ( Flag[IndexOfFlag["P"]] == 1 )						   //Conditionally returns control from a sub-routine through
        RET ();                                                                            //a RET call if parity flag is currently 1
}

void MainWin::RM ()
{
	if ( Flag[IndexOfFlag["S"]] == 1 )						   //Conditionally returns control from a sub-routine through
    	RET ();                                                                            //a RET call if sign flag is currently 1
}

void MainWin::RC ()
{
	if ( Flag[IndexOfFlag["CY"]] == 1 )					          //Conditionally returns control from a sub-routine through
    	RET ();                                                                           //a RET call if carry flag is currently 1
}


/************************** CONTRIBUTED BY RAJESH ***************************
*****************************************************************************/
void MainWin::STC()
{
    Flag[IndexOfFlag["CY"]] = 1;
    UpdateAF();
}

void MainWin::JMP(int a)
{            
    Register[IndexOfRegister["PC"]] = a;
}

void MainWin::PCHL()
{
    Register[IndexOfRegister["PC"]] = Memory[Register[IndexOfRegister["HL"]]];
}

void MainWin::RLC()
{
    std::string FBin = "00000000";
    int K = 7;
    unsigned FVal = Register[IndexOfRegister["A"]];
    while(FVal)
    {
        if(FVal%2)
            FBin[K] = '1';
        FVal /= 2;
        K--;
    }
    Flag[IndexOfFlag["CY"]] = FBin[0] - '0';
    K = 0;
    while(K < 7)
    {
        FBin[K] = FBin[K+1];
        ++K;
    }
    FBin[7] = Flag[IndexOfFlag["CY"]] + '0';

    ChangedRegs.push_back("A");

    Register[IndexOfRegister["A"]] = 0;
    K = 7;
    int Pow2 = 1;
    while(K > -1)
    {
        if(FBin[K]-'0')
            Register[IndexOfRegister["A"]] += Pow2;
        Pow2 <<= 1;
        K--;
    }
}

void MainWin::RRC()
{
    std::string FBin = "00000000";
    int K = 7;
    int FVal = Register[IndexOfRegister["A"]];
    while(FVal)
    {
        if(FVal%2)
            FBin[K] = '1';
        FVal /= 2;
        K--;
    }

    Flag[IndexOfFlag["CY"]] = FBin[7] - '0';
    K = 7;
    while(K > 0)
    {
        FBin[K] = FBin[K-1];
        --K;
    }
    FBin[0] = Flag[IndexOfFlag["CY"]] + '0';
    ChangedRegs.push_back("A");

    Register[IndexOfRegister["A"]] = 0;
    K = 7;
    int Pow2 = 1;
    while(K > -1)
    {
        if(FBin[K]-'0')
            Register[IndexOfRegister["A"]] += Pow2;
        Pow2 <<= 1;
        K--;
    }
}


/*********************** CONTRIBUTED BY PAWAN NAGWANI ***********************
*****************************************************************************/
// Calling functions
bool MainWin::Call(std::string a)
{
    if(a == "RZ")
            RZ();
    else if(a == "RP")
            RP();
    else if(a == "RPO")
            RPO();
    else if(a == "RPE")
            RPE();
    else if(a == "RM")
            RM();
    else if(a == "RC")
            RC();
    else if(a == "STC")
            STC();
    else if(a == "PCHL")
            PCHL();
    else if(a == "XCHG")
            XCHG();
    else if(a == "NOP")
            NOP();
    else if(a == "XTHL")
            XTHL();
    else if(a == "RET")
            RET();
    else if(a == "RNC")
            RNC();
    else if(a == "RNZ")
            RNZ();
    else if(a == "RAR")
            RAR();
    else if(a == "RAL")
            RAL();
    else if(a == "CMA")
            CMA();
    else if(a == "CMC")
            CMC();
    else if(a == "RRC")
            RRC();
    else if(a == "RLC")
            RLC();
    else
        return false;
    return true;
}

bool MainWin::Call(std::string a, int p)
{
    if(a == "ACI")
            ACI(p);
    else if(a == "ADI")
            ADI(p);
    else if(a == "ANI")
            ANI(p);
    else if(a == "CALL")
            CALL(p);
    else if(a == "CC")
            CC(p);
    else if(a == "CM")
            CM(p);
    else if(a == "CNC")
            CNC(p);
    else if(a == "CNZ")
            CNZ(p);
    else if(a == "CP")
            CP(p);
    else if(a == "CPE")
            CPE(p);
    else if(a == "CPI")
            CPI(p);
    else if(a == "CPO")
            CPO(p);
    else if(a == "CZ")
            CZ(p);
    else if(a == "JC")
            JC(p);
    else if(a == "JM")
            JM(p);
    else if(a == "JNC")
            JNC(p);
    else if(a == "JNZ")
            JNZ(p);
    else if(a == "JP")
            JP(p);
    else if(a == "JPE")
            JPE(p);
    else if(a == "JPO")
            JPO(p);
    else if(a == "LDA")
            LDA(p);
    else if(a == "LHLD")
            LHLD(p);
    else if(a == "ORI")
            ORI(p);
    else if(a == "SBI")
            SBI(p);
    else if(a == "SUI")
            SUI(p);
    else if(a == "XRI")
            XRI(p);
    else
        return false;
    return true;
}

bool MainWin::Call(std::string a, std::string x)
{
    if(a == "ADD")
            ADD(x);
    else if(a == "ADC")
            ADC(x);
    else if(a == "ANA")
            ANA(x);
    else if(a == "XRA")
            XRA(x);
    else if(a == "ORA")
            ORA(x);
    else if(a == "POP")
            POP(x);
    else if(a == "PUSH")
            PUSH(x);
    else if(a == "DCR")
            DCR(x);
    else if(a == "DCX")
            DCX(x);
    else if(a == "DAD")
            DAD(x);
    else if(a == "CMP")
            CMP(x);
    else if(a == "LDAX")
            LDAX(x);
    else if(a == "SBB")
            SBB(x);
    else if(a == "SUB")
            SUB(x);
    else
        return false;
    return true;
}

bool MainWin::Call(std::string a, std::string x, int p)
{
    if(a == "LXI")
            LXI(x,p);
    else if(a == "MVI")
            MVI(x,p);
    else
        return false;
    return true;	
}

bool MainWin::Call(std::string a, std::string x, std::string y)
{
    if(a == "MOV")
            MOV(x,y);
    else
        return false;
    return true;
}


//SIMCALLS
bool MainWin::SimCall(std::string a)
{
    if(a == "RZ")
            return true;
    else if(a == "RP")
            return true;
    else if(a == "RPO")
            return true;
    else if(a == "RPE")
            return true;
    else if(a == "RM")
            return true;
    else if(a == "RC")
            return true;
    else if(a == "STC")
            return true;
    else if(a == "PCHL")
            return true;
    else if(a == "XCHG")
            return true;
    else if(a == "NOP")
            return true;
    else if(a == "XTHL")
            return true;
    else if(a == "RET")
            return true;
    else if(a == "RNC")
            return true;
    else if(a == "RNZ")
            return true;
    else if(a == "RAR")
            return true;
    else if(a == "RAL")
            return true;
    else if(a == "CMA")
            return true;
    else if(a == "CMC")
            return true;
    else if(a == "RRC")
            return true;
    else if(a == "RLC")
            return true;
    else
        return false;
}

bool MainWin::SimCall(std::string a, int p)
{
    if(a == "ACI")
            return true;
    else if(a == "ADI")
            return true;
    else if(a == "ANI")
            return true;
    else if(a == "CALL")
            return true;
    else if(a == "CC")
            return true;
    else if(a == "CM")
            return true;
    else if(a == "CNC")
            return true;
    else if(a == "CNZ")
            return true;
    else if(a == "CP")
            return true;
    else if(a == "CPE")
            return true;
    else if(a == "CPI")
            return true;
    else if(a == "CPO")
            return true;
    else if(a == "CZ")
            return true;
    else if(a == "JC")
            return true;
    else if(a == "JM")
            return true;
    else if(a == "JNC")
            return true;
    else if(a == "JNZ")
            return true;
    else if(a == "JP")
            return true;
    else if(a == "JPE")
            return true;
    else if(a == "JPO")
            return true;
    else if(a == "LDA")
            return true;
    else if(a == "LHLD")
            return true;
    else if(a == "ORI")
            return true;
    else if(a == "SBI")
            return true;
    else if(a == "SUI")
            return true;
    else if(a == "XRI")
            return true;
    else
        return false;
}

bool MainWin::SimCall(std::string a, std::string x)
{
    if(a == "ADD")
            return true;
    else if(a == "ADC")
            return true;
    else if(a == "ANA")
            return true;
    else if(a == "XRA")
            return true;
    else if(a == "ORA")
            return true;
    else if(a == "POP")
            return true;
    else if(a == "PUSH")
            return true;
    else if(a == "DCR")
            return true;
    else if(a == "DCX")
            return true;
    else if(a == "DAD")
            return true;
    else if(a == "CMP")
            return true;
    else if(a == "LDAX")
            return true;
    else if(a == "SBB")
            return true;
    else if(a == "SUB")
            return true;
    else
        return false;
}

bool MainWin::SimCall(std::string a, std::string x, int p)
{
    if(a == "LXI")
            return true;
    else if(a == "MVI")
            return true;
    else
        return false;
}

bool MainWin::SimCall(std::string a, std::string x, std::string y)
{
    if(a == "MOV")
        return true;
    else
        return false;
}
