#include <QtGui/QApplication>
#include "mainwin.h"

void mySleep(float sec) // clock_t is a like typedef unsigned int clock_t. Use clock_t instead of integer in this context
{
    clock_t endwait;
    endwait = clock () + sec * CLOCKS_PER_SEC ;
    while (clock() < endwait);
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QSplashScreen *splash = new QSplashScreen;
    QString SplashFile = "./SPLASH/splash-";
    QString P, Q, N, K = ".jpg";
    Qt::Alignment topRight = Qt::AlignHCenter | Qt::AlignBottom;
    QLabel *MainLabel, *SubLabel;
    QPalette Z, ZZ;
    QTime TX(0,0,0);
    QFont X, XX;
    int SI, MaxIndex, DelIndex;

    splash->setPixmap(QPixmap(SplashFile+N.setNum(0)+K));

    Z.setColor(QPalette::Foreground,Qt::red);
    Z.setColor(QPalette::Text,Qt::red);
    X.setPointSize(54);
    X.setFamily("Unexplored Galaxies BRK");

    ZZ.setColor(QPalette::Foreground,Qt::white);
    ZZ.setColor(QPalette::Text,Qt::white);
    XX.setPointSize(14);
    XX.setFamily("Bitstream Charter");

    SubLabel = new QLabel;
    SubLabel->setParent(splash);
    SubLabel->setText("CS 101 Project by Lab Batch 22");
    SubLabel->resize(270 , 30);
    SubLabel->setPalette(ZZ);
    SubLabel->setFont(XX);
    SubLabel->move((splash->size().width() - SubLabel->size().width())/2,100 + (splash->size().height()-(40 + SubLabel->size().height()))/2);
    SubLabel->setVisible(true);

    MainLabel = new QLabel;
    MainLabel->setParent(splash);
    MainLabel->setText("Intel  8085\n Simulator");
    MainLabel->resize(320 , 140);
    MainLabel->setPalette(Z);
    MainLabel->setFont(X);
    MainLabel->move((splash->size().width()-MainLabel->size().width())/2,(splash->size().height()-(40 + MainLabel->size().height()))/2);
    MainLabel->setVisible(true);

    MainWin w;
    qsrand(TX.secsTo(TX.currentTime()));
    SI = sizeof(int);
    MaxIndex = 0;

    DelIndex = 600 + (rand() % 1000);
    memset(w.Memory,SI,DelIndex);
    memset(w.Stack,SI,DelIndex);
    MaxIndex += DelIndex;

    splash->show();
    Q = " %";
    for(int k = 1; k < 39; ++k)
    {
        P = "Loading Simulator Window ... ";
        P += N.setNum((100*k)/39);
        P += Q;

        splash->showMessage(QObject::tr(P.toUtf8()), topRight, Qt::yellow);
        splash->setPixmap(QPixmap(SplashFile+N.setNum(k)+K));

        mySleep(0.02 + ((rand()%5)/100.0));
        DelIndex = 600 + (rand() % 1000);
        memset(w.Memory+MaxIndex,SI,DelIndex);
        memset(w.Stack+MaxIndex,SI,DelIndex);
        MaxIndex += DelIndex;
    }

    w.setWindowOpacity(0.0);
    splash->setPixmap(QPixmap(SplashFile+N.setNum(0)+K));
    w.move((qApp->desktop()->width()-w.size().width())/2,(qApp->desktop()->height()-w.size().height())/2);

    for(int k = 1; k < 39; ++k)
    {
        P = "Initializing 8085 Engine ... ";
        P += N.setNum((100*k)/39);
        P += Q;
        splash->showMessage(QObject::tr(P.toUtf8()), topRight, Qt::yellow);
        splash->setPixmap(QPixmap(SplashFile+N.setNum(k)+K));

        mySleep(0.02 + ((rand()%5)/100.0));
        DelIndex = 600 + (rand() % 1000);
        if(DelIndex + MaxIndex > 65535)
            DelIndex = 65535-MaxIndex;
        if(DelIndex <= 0)
            continue;
        memset(w.Memory+MaxIndex,SI,DelIndex);
        memset(w.Stack+MaxIndex,SI,DelIndex);
        MaxIndex += DelIndex;
    }

    MainLabel->setVisible(false);
    SubLabel->setVisible(false);
    splash->showMessage(QObject::tr(""),topRight,Qt::black);

    int AnimSteps = 101;
    float dX = (w.size().width() - splash->size().width()) / (AnimSteps*2.0);
    float dY = (w.size().height() - splash->size().height()) / (AnimSteps*2.0);
    while(AnimSteps--)
    {
        splash->setWindowOpacity(AnimSteps/100.0);
        splash->move(int(splash->pos().x() - dX), int(splash->pos().y() - dY));
        splash->resize(int(splash->size().width() + dX*2.0), int(splash->size().height() + dY*2.0));
        mySleep(0.005);
        qApp->processEvents();
    }
    splash->move(w.pos().x(), w.pos().y());
    splash->resize(w.size().width(), w.size().height());

    w.show();
    AnimSteps = 100;
    while(AnimSteps)
    {
        AnimSteps -= 4;
        w.setWindowOpacity((100-AnimSteps)/100.0);
        mySleep(0.005);
        qApp->processEvents();
    }
    splash->finish(&w);
    delete splash;

    return a.exec();
}
