#include "mainwin.h"
#include "ui_mainwin.h"

int CurrentLine, ErrorCounter;
int OldError;
int MemLoc;

ifstream InputFile;

void MainWin::ErrorFound(std::string ErrorCode)
{
    ALL_ERROR_LINES.push_back(CurrentLine);
    ALL_ERRORS.push_back(ErrorCode);
    ++ErrorCounter;
}

MainWin::MainWin(QWidget *parent) : QMainWindow(parent), ui(new Ui::MainWin)
{
    int K;
    QString Z;
    QStringList HHL;

    MemLoc = 0;
    CodeStatus = "";

    ui->setupUi(this);
    string J;

    this->Initialize();
    this->IndexOfRegister["PSW"] = 13;

    for(K = 0; K < 20; ++K)
    {
        MemItems[K][0].setTextAlignment(Qt::AlignHCenter);
        MemItems[K][1].setTextAlignment(Qt::AlignHCenter);

        MemItems[K][0].setText("NOP");
        ui->tableOpCode->setItem(K, 0, *(MemItems+K));

        Z.clear();
        Z.setNum(ui->sldMem->value()+K,16);
        HHL << Z;

        MemItems[K][1].setText("0");
        ui->tableOpCode->setItem(K, 1, *(MemItems+K)+1);
    }
    ui->tableOpCode->setVerticalHeaderLabels(HHL);

    for(K = 0; K < 7; ++K)
    {
        RegItems[K].setTextAlignment(Qt::AlignHCenter);
        RegItems[K].setText("0");
        ui->tableReg->setItem(K, 0, RegItems+K);
    }

    for(K = 0; K < 5; ++K)
    {
        FlaItems[K].setTextAlignment(Qt::AlignHCenter);
        FlaItems[K].setText("0");
        ui->tableFlag->setItem(K, 0, FlaItems+K);
    }

    for(K = 0; K < 7; ++K)
    {
        StaItems[K][0].setTextAlignment(Qt::AlignHCenter);
        StaItems[K][1].setTextAlignment(Qt::AlignHCenter);

        if(((Register[IndexOfRegister["SP"]]+K-3) < 65535) && ((Register[IndexOfRegister["SP"]]+K-3) >= 0))
        {
            Z.setNum(Register[IndexOfRegister["SP"]]+K-3,16);
            StaItems[K][0].setText(Z);
            ui->tableStack->setItem(K, 0, *(StaItems+K));

            Z.setNum(Stack[Register[IndexOfRegister["SP"]]+K-3],16);
            StaItems[K][1].setText(Z);
            ui->tableStack->setItem(K, 1, *(StaItems+K)+1);
        }
        else
        {
            StaItems[K][0].setText("<undefined>");
            ui->tableStack->setItem(K, 0, *(StaItems+K));

            StaItems[K][1].setText("<undefined>");
            ui->tableStack->setItem(K, 1, *(StaItems+K)+1);
        }
    }

    HHL.clear();
    HHL << "8085 Mnemonics / Data" << "OpCodes";
    ui->tableOpCode->setHorizontalHeaderLabels(HHL);

    ui->tableOpCode->setColumnWidth(0,245);
    ui->tableOpCode->setColumnWidth(1,165);
    ui->tableStack->setColumnWidth(0,130);

    ui->tableOpCode->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->tableFlag->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->tableReg->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->tableStack->setEditTriggers(QAbstractItemView::NoEditTriggers);

    ui->btnNext->setEnabled(false);
    this->FileName = "Untitled.8085";
    this->FirstSave = true;
    ui->lblFileName->setText(this->FileName);
    ui->action_Undo->setEnabled(false);
    ui->action_Redo->setEnabled(false);

    connect(&Sim, SIGNAL(timeout()), this, SLOT(ExecuteStep()));
    Sim.stop();
}

MainWin::~MainWin()
{
    delete ui;
}

void MainWin::on_action_About_triggered()
{
    MyAbout = new about;
    MyAbout->show();
}

void MainWin::on_actionExit_triggered()
{
    this->close();
}

void MainWin::on_action_Open_triggered()
{
    int MRet;
    if(ui->lblFileName->text().right(4) == " [*]")
    {
        MRet = QMessageBox(QString("Unsaved file"), QString("The active file <") + ui->lblFileName->text().right(ui->lblFileName->text().length()-4) + \
                           QString(" >is unsaved.\nYou would lose any unsaved changes in this file.\n\nAre you sure you want to create a new file ?"), \
                           QMessageBox::Critical,QMessageBox::Cancel,QMessageBox::Save,QMessageBox::Discard).exec();
        if(MRet == QMessageBox::Cancel)
            return;
        else if(MRet == QMessageBox::Save)
            on_action_Save_triggered();
    }
    QString FileToBeOpened = QFileDialog::getOpenFileName(this, tr(".: OPENING 8085 FILE :."), "/home", tr("8085 Instructions [*.8085] (*.8085)"));

    if(FileToBeOpened.length() == 0)
        return;

    QFile OpenF;
        OpenF.setFileName(FileToBeOpened);
        if(!OpenF.open(QIODevice::ReadOnly | QIODevice::Text))
            return;
        QByteArray Xline = OpenF.readAll();
        ui->txtSource->clear();
        ui->txtSource->appendPlainText(Xline);
    OpenF.close();

    this->CodeStatus = ui->txtSource->toPlainText();
    this->FirstSave = false;
    this->FileName = FileToBeOpened;
    ui->lblFileName->setText(FileToBeOpened);
    ui->action_Undo->setEnabled(false);
    ui->action_Redo->setEnabled(false);
}

bool MainWin::on_action_Save_triggered()
{
    if(this->FirstSave)
        return on_actionSave_As_triggered();
    else
    {
        QString FileToBeSaved;
        if(ui->lblFileName->text().right(4) == " [*]")
            FileToBeSaved = ui->lblFileName->text().left(ui->lblFileName->text().length()-4);
        else
            FileToBeSaved = ui->lblFileName->text().left(ui->lblFileName->text().length());
        if(FileToBeSaved.length() == 0)
            return false;

        QFile SaveF(FileToBeSaved);
        if(!SaveF.open(QIODevice::WriteOnly | QIODevice::Text))
            return false;
        QByteArray Xline = ui->txtSource->toPlainText().toUtf8();
        SaveF.write(Xline);
        SaveF.close();
        this->FileName = FileToBeSaved;
        ui->lblFileName->setText(FileToBeSaved);
        this->CodeStatus = ui->txtSource->toPlainText();
        return true;
    }
}

void MainWin::on_action_Undo_triggered()
{
    ui->txtSource->undo();
}

void MainWin::on_txtSource_redoAvailable(bool b)
{
    ui->action_Redo->setEnabled(b);
}

void MainWin::on_txtSource_undoAvailable(bool b)
{
    ui->action_Undo->setEnabled(b);
}

void MainWin::on_action_Redo_triggered()
{
    ui->txtSource->redo();
}

void MainWin::on_action_Select_All_triggered()
{
    ui->txtSource->selectAll();
}

void MainWin::on_action_Cut_triggered()
{
    ui->txtSource->cut();
}

void MainWin::on_actionC_opy_triggered()
{
    ui->txtSource->copy();
}

void MainWin::on_action_Paste_triggered()
{
    ui->txtSource->paste();
}

void MainWin::on_txtSource_textChanged()
{
    if(ui->txtSource->toPlainText() != CodeStatus)
    {
        ui->action_Save->setEnabled(true);
        ui->lblFileName->setText(this->FileName+" [*]");
    }
    else
    {
        ui->action_Save->setEnabled(false);
        ui->lblFileName->setText(this->FileName);
    }
}

bool MainWin::on_actionSave_As_triggered()
{
    QString FileToBeSaved = QFileDialog::getSaveFileName(this, tr(".: SAVING 8085 FILE :."), "/home", tr("8085 Instructions [*.8085] (*.8085)"));
    if(FileToBeSaved.length() == 0)
        return false;

    if(FileToBeSaved.right(5) != ".8085")
        FileToBeSaved += ".8085";
    QFile SaveF(FileToBeSaved);
    this->FileName = FileToBeSaved;
    if(!SaveF.open(QIODevice::WriteOnly | QIODevice::Text))
        return false;
    QByteArray Xline = ui->txtSource->toPlainText().toUtf8();
    SaveF.write(Xline);
    SaveF.close();
    this->FirstSave = false;
    ui->lblFileName->setText(FileToBeSaved);
    this->CodeStatus = ui->txtSource->toPlainText();
    return true;
}

void MainWin::on_actionRefresh_triggered()
{
    on_sldMem_sliderReleased();

    qApp->processEvents();
    int K;
    QString Z;

    for(K = 0; K < 7; ++K)
    {
        Z.setNum(Register[K],16);
        ui->tableReg->item(K, 0)->setText(Z);
    }

    for(K = 0; K < 5; ++K)
    {
        Z.setNum(Flag[K],16);
        ui->tableFlag->item(K, 0)->setText(Z);
    }

    for(K = 0; K < 7; ++K)
    {
        if(((Register[IndexOfRegister["SP"]]+K-3) < 65535) && ((Register[IndexOfRegister["SP"]]+K-3) >= 0))
        {
           Z.setNum(Register[IndexOfRegister["SP"]]+K-3,16);
           ui->tableStack->item(K, 0)->setText(Z);

           Z.setNum(Stack[Register[IndexOfRegister["SP"]]+K-3],16);
           ui->tableStack->item(K, 1)->setText(Z);
        }
        else
        {
            Z = "<undefined>";
            ui->tableStack->item(K, 0)->setText(Z);
            ui->tableStack->item(K, 1)->setText(Z);
        }
    }
}

void MainWin::on_action_New_triggered()
{
    int MRet;
    if(ui->lblFileName->text().right(4) == " [*]")
    {
        MRet = QMessageBox(QString("Unsaved file"), QString("The active file <") + ui->lblFileName->text().left(ui->lblFileName->text().length()-4) + \
                           QString(" >is unsaved.\nYou would lose any unsaved changes in this file.\n\nAre you sure you want to create a new file ?"), \
                           QMessageBox::Critical,QMessageBox::Cancel,QMessageBox::Save,QMessageBox::Discard).exec();
        if(MRet == QMessageBox::Cancel)
            return;
        else if(MRet == QMessageBox::Save)
            on_action_Save_triggered();
    }
    ui->txtSource->setPlainText("");
    this->CodeStatus = "";
    this->FileName = "Untitled.8085";
    ui->lblFileName->setText(this->FileName);
    this->FirstSave = true;
}

void MainWin::on_sldMem_sliderReleased()
{
    int K, X;
    int J = ui->sldMem->value();
    QString Z;
    QStringList VHL;

    if(J+19 < MemLoc)
    {
        for(K = 0; K <= 20; ++K)
        {
            if(MemInstr[J+K] != "")
                Z.fromStdString(MemInstr[J+K]);
            else
                Z = "";
            ui->tableOpCode->item(K, 0)->setText(Z);

            Z.setNum(Memory[J+K],16);
            ui->tableOpCode->item(K, 1)->setText(Z);

            Z.setNum(J+K,16);
            VHL << Z;
        }
    }
    else if(J < MemLoc)
    {
        X = MemLoc-J;
        for(K = 0; K < X; ++K)
        {
            if(!MemInstr[J+K].size())
                Z.fromStdString(MemInstr[J+K]);
            else
                Z = "";
            ui->tableOpCode->item(K, 0)->setText(Z);

            Z.setNum(Memory[J+K],16);
            ui->tableOpCode->item(K, 1)->setText(Z);

            Z.setNum(J+K,16);
            VHL << Z;
        }
        for(K = X; K < 20; ++K)
        {
            ui->tableOpCode->item(K, 0)->setText("NOP");
            ui->tableOpCode->item(K, 1)->setText("0");

            Z.setNum(J+K,16);
            VHL << Z;
        }
    }
    else
    {
        for(K = 0; K < 20; ++K)
        {
            ui->tableOpCode->item(K, 0)->setText("NOP");
            ui->tableOpCode->item(K, 1)->setText("0");

            Z.setNum(J+K,16);
            VHL << Z;
        }
    }
    ui->tableOpCode->setVerticalHeaderLabels(VHL);
}

void MainWin::on_sldMem_valueChanged(int value)
{
    on_sldMem_sliderReleased();
}

void CAPITALIZE(char* TheLine)
{
    int j = strlen(TheLine);
    for(int i = 0; i < j; ++i)
    {
        if(TheLine[i] >= 'a' && TheLine[i] <= 'z')
            TheLine[i] -= 32;
    }
}

void MainWin::AnalyzeALine(char* TheLine)
{
    std::string Command, Param, Param_2;
    int x, i, p, f, j = strlen(TheLine);

    CAPITALIZE(TheLine);
    Command = TheLine[0];

    for(i = 1; (TheLine[i] != ' ') && (i < j); ++i)
        Command += TheLine[i];

    if(!binary_search(Instruction.begin(), Instruction.end(),Command))
            return;
    if(Parameters[Command] == 0)
    {
        if(i != j)
            return;
        else
        {
            if(!Call(Command))
                return;
        }
    }
    else if(Parameters[Command] == 1)
    {
        Param = "";

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            return;
        else
        {
            if(i+1 != j)
                return;
            else
            {
                f = 0;
                f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
                if(f)
                    return;
                else
                {
                    Param = TheLine[i];
                    ChangedRegs.push_back(Param);
                    if(!Call(Command, Param))
                        return;
                }
            }
        }
    }
    else if(Parameters[Command] == 2)
    {
        x = 0;
        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            return;
        else if(TheLine[i] == '0')
        {
            if(TheLine[i+1] == 'X')
            {
                if(j > (i+4))
                    return;
                else if(j == (i+2))
                    return;
                else if(j == (i+3))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    if(f)
                        return;
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!Call(Command, x))
                            return;
                    }
                }
                else
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    if(f)
                        return;
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!Call(Command, x))
                            return;
                    }
                }
            }
            else
                return;
        }
    }
    else if(Parameters[Command] == 3)
    {
        x = 0;
        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            return;
        else if(TheLine[i] == '0')
        {
            if(TheLine[i+1] == 'X')
            {
                if(j > (i+6))
                    return;
                else if(j == (i+2))
                    return;
                else if(j == (i+3))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    if(f)
                        return;
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!Call(Command, x))
                            return;
                    }
                }
                else if(j == (i+4))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    if(f)
                        return;
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!Call(Command, x))
                            return;
                    }
                }
                else if(j == (i+5))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                    if(f)
                        return;
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!Call(Command, x))
                            return;
                    }
                }
                else
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+5] >= '0' && TheLine[i+5] <= '9') || (TheLine[i+5] >= 'A' && TheLine[i+5] <= 'F')) ? 0 : 1;
                    if(f)
                        return;
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!Call(Command, x))
                            return;
                    }
                }
            }
            else
                return;
        }
    }
    else if(Parameters[Command] == 4)
    {
        Param = "";
        Param_2 = "";

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            return;
        else
        {
            p = i;
            f = 0;
            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
            if(f)
                return;
            else
            {
                Param = TheLine[p];
                ChangedRegs.push_back(Param);
                i++;
                while(TheLine[i] == ' ')
                    ++i;
                if(TheLine[i] != ',')
                    return;
                else
                {
                    i++;
                    while(TheLine[i] == ' ')
                        ++i;
                    if(i == j)
                        return;
                    else
                    {
                        if(i+1 != j)
                            return;
                        else
                        {
                            f = 0;
                            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
                            if(f)
                                return;
                            else
                            {
                                Param = TheLine[p];
                                Param_2 = TheLine[i];
                                if(!Call(Command, Param, Param_2))
                                    return;
                            }
                        }
                    }
                }
            }
        }
    }
    else if(Parameters[Command] == 5)
    {
        Param = "";
        x = 0;

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            return;
        else
        {
            p = i;
            f = 0;
            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
            if(f)
                return;
            else
            {
                Param = TheLine[p];
                ChangedRegs.push_back(Param);
                i++;
                while(TheLine[i] == ' ')
                    ++i;
                if(TheLine[i] != ',')
                    return;
                else
                {
                    i++;
                    while(TheLine[i] == ' ')
                        ++i;
                    if(i == j)
                        return;
                    else if(TheLine[i] == '0')
                    {
                        if(TheLine[i+1] == 'X')
                        {
                            if(j > (i+4))
                                return;
                            else if(j == (i+2))
                                return;
                            else if(j == (i+3))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                if(f)
                                    return;
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    if(!Call(Command, x))
                                        return;
                                }
                            }
                            else
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                if(f)
                                    return;
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!Call(Command, Param, x))
                                        return;
                                }
                            }
                        }
                        else
                            return;
                    }
                }
            }
        }
    }
    else
    {
        Param = "";
        x = 0;

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            return;
        else
        {
            p = i;
            f = 0;
            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
            if(f)
                return;
            else
            {
                Param = TheLine[p];
                ChangedRegs.push_back(Param);
                i++;
                while(TheLine[i] == ' ')
                    ++i;
                if(TheLine[i] != ',')
                    return;
                else
                {
                    i++;
                    while(TheLine[i] == ' ')
                        ++i;
                    if(i == j)
                        return;
                    else if(TheLine[i] == '0')
                    {
                        if(TheLine[i+1] == 'X')
                        {
                            if(j > (i+6))
                                return;
                            else if(j == (i+2))
                                return;
                            else if(j == (i+3))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                if(f)
                                    return;
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    if(!Call(Command, x))
                                        return;
                                }
                            }
                            else if(j == (i+4))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                if(f)
                                    return;
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!Call(Command, Param, x))
                                        return;
                                }
                            }
                            else if(j == (i+5))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                                if(f)
                                    return;
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!Call(Command, Param, x))
                                        return;
                                }
                            }
                            else
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+5] >= '0' && TheLine[i+5] <= '9') || (TheLine[i+5] >= 'A' && TheLine[i+5] <= 'F')) ? 0 : 1;
                                if(f)
                                    return;
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!Call(Command, Param, x))
                                        return;
                                }
                            }
                        }
                        else
                            return;
                    }
                }
            }
        }
    }
}

void MainWin::GenOpCodeForLine(char* TheLine)
{
    std::string Command, Param, Param_2;
    int x, i, p, f, j = strlen(TheLine);

    Command = TheLine[0];

    for(i = 1; (TheLine[i] != ' ') && (i < j); ++i)
        Command += TheLine[i];

    if(!binary_search(Instruction.begin(), Instruction.end(),Command))
        ErrorFound("ER_05");
    else if(Parameters[Command] == 0)
    {
        if(i != j)
            ErrorFound("ER_00");
        else
        {
            if(!SimCall(Command))
                ErrorFound("ER_05");
            else
            {
                Memory[MemLoc] = this->InstCode[Command];
                MemInstr[MemLoc] = Command;
                MemLoc++;
            }
        }
    }
    else if(Parameters[Command] == 1)
    {
        Param = "";

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            ErrorFound("ER_01");
        else
        {
            if(i+1 != j)
                ErrorFound("ER_03");
            else
            {
                f = 0;
                f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
                if(f)
                    ErrorFound("ER_03");
                else
                {
                    Param = TheLine[i];
                    ChangedRegs.push_back(Param);
                    if(!SimCall(Command, Param))
                        ErrorFound("ER_05");
                    else
                    {
                        Command += " ";
                        Command += Param;
                        Memory[MemLoc] = this->InstCode[Command];
                        MemInstr[MemLoc] = Command;
                        MemLoc++;
                    }
                }
            }
        }
    }
    else if(Parameters[Command] == 2)
    {
        x = 0;
        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            ErrorFound("ER_01");
        else if(TheLine[i] == '0')
        {
            if(TheLine[i+1] == 'X')
            {
                if(j > (i+4))
                    ErrorFound("ER_02");
                else if(j == (i+2))
                    ErrorFound("ER_02");
                else if(j == (i+3))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    if(f)
                        ErrorFound("ER_02");
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!SimCall(Command, x))
                            ErrorFound("ER_05");
                        else
                        {
                            Memory[MemLoc] = this->InstCode[Command];
                            MemInstr[MemLoc] = Command;
                            MemLoc++;
                        }
                    }
                }
                else
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    if(f)
                        ErrorFound("ER_02");
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!SimCall(Command, x))
                            ErrorFound("ER_05");
                        else
                        {
                            Memory[MemLoc] = this->InstCode[Command];
                            MemInstr[MemLoc] = Command;
                            MemLoc++;
                        }
                    }
                }
            }
            else
                ErrorFound("ER_02");
        }
    }
    else if(Parameters[Command] == 3)
    {
        x = 0;
        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            ErrorFound("ER_01");
        else if(TheLine[i] == '0')
        {
            if(TheLine[i+1] == 'X')
            {
                if(j > (i+6))
                    ErrorFound("ER_06");
                else if(j == (i+2))
                    ErrorFound("ER_06");
                else if(j == (i+3))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    if(f)
                        ErrorFound("ER_06");
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!SimCall(Command, x))
                            ErrorFound("ER_05");
                        else
                        {
                            Memory[MemLoc] = this->InstCode[Command];
                            MemInstr[MemLoc] = Command;
                            MemLoc++;
                        }
                    }
                }
                else if(j == (i+4))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    if(f)
                        ErrorFound("ER_06");
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!SimCall(Command, x))
                            ErrorFound("ER_05");
                        else
                        {
                            Memory[MemLoc] = this->InstCode[Command];
                            MemInstr[MemLoc] = Command;
                            MemLoc++;
                        }
                    }
                }
                else if(j == (i+5))
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                    if(f)
                        ErrorFound("ER_06");
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!SimCall(Command, x))
                            ErrorFound("ER_05");
                        else
                        {
                            Memory[MemLoc] = this->InstCode[Command];
                            MemInstr[MemLoc] = Command;
                            MemLoc++;
                        }
                    }
                }
                else
                {
                    f = 0;
                    f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                    f |= ((TheLine[i+5] >= '0' && TheLine[i+5] <= '9') || (TheLine[i+5] >= 'A' && TheLine[i+5] <= 'F')) ? 0 : 1;
                    if(f)
                        ErrorFound("ER_06");
                    else
                    {
                        sscanf(TheLine+i+2, "%X", &x);
                        if(!SimCall(Command, x))
                            ErrorFound("ER_05");
                        else
                        {
                            Memory[MemLoc] = this->InstCode[Command];
                            MemInstr[MemLoc] = Command;
                            MemLoc++;
                        }
                    }
                }
            }
            else
                ErrorFound("ER_06");
        }
    }
    else if(Parameters[Command] == 4)
    {
        Param = "";
        Param_2 = "";

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            ErrorFound("ER_01");
        else
        {
            p = i;
            f = 0;
            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
            if(f)
                ErrorFound("ER_03");
            else
            {
                Param = TheLine[p];
                ChangedRegs.push_back(Param);
                i++;
                while(TheLine[i] == ' ')
                    ++i;
                if(TheLine[i] != ',')
                    ErrorFound("ER_01");
                else
                {
                    i++;
                    while(TheLine[i] == ' ')
                        ++i;
                    if(i == j)
                        ErrorFound("ER_01");
                    else
                    {
                        if(i+1 != j)
                            ErrorFound("ER_03");
                        else
                        {
                            f = 0;
                            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
                            if(f)
                                ErrorFound("ER_03");
                            else
                            {
                                Param = TheLine[p];
                                Param_2 = TheLine[i];
                                if(!SimCall(Command, Param, Param_2))
                                    ErrorFound("ER_05");
                                else
                                {
                                    Command += " ";
                                    Command += Param;
                                    Command += ",";
                                    Command += Param_2;
                                    Memory[MemLoc] = this->InstCode[Command];
                                    MemInstr[MemLoc] = Command;
                                    MemLoc++;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    else if(Parameters[Command] == 5)
    {
        Param = "";
        x = 0;

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            ErrorFound("ER_01");
        else
        {
            p = i;
            f = 0;
            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
            if(f)
                ErrorFound("ER_03");
            else
            {
                Param = TheLine[p];
                ChangedRegs.push_back(Param);
                i++;
                while(TheLine[i] == ' ')
                    ++i;
                if(TheLine[i] != ',')
                    ErrorFound("ER_01");
                else
                {
                    i++;
                    while(TheLine[i] == ' ')
                        ++i;
                    if(i == j)
                        ErrorFound("ER_01");
                    else if(TheLine[i] == '0')
                    {
                        if(TheLine[i+1] == 'X')
                        {
                            if(j > (i+4))
                                ErrorFound("ER_02");
                            else if(j == (i+2))
                                ErrorFound("ER_02");
                            else if(j == (i+3))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                if(f)
                                    ErrorFound("ER_02");
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    if(!SimCall(Command, Param, x))
                                        ErrorFound("ER_05");
                                    else
                                    {
                                        Command += " ";
                                        Command += Param;
                                        Memory[MemLoc] = this->InstCode[Command];
                                        MemInstr[MemLoc] = Command;
                                        MemLoc++;
                                    }
                                }
                            }
                            else
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                if(f)
                                    ErrorFound("ER_02");
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!SimCall(Command, Param, x))
                                        ErrorFound("ER_05");
                                    else
                                    {
                                        Command += " ";
                                        Command += Param;
                                        Memory[MemLoc] = this->InstCode[Command];
                                        MemInstr[MemLoc] = Command;
                                        MemLoc++;
                                    }
                                }
                            }
                        }
                        else
                            ErrorFound("ER_02");
                    }
                }
            }
        }
    }
    else
    {
        Param = "";
        x = 0;

        while(TheLine[i] == ' ')
            ++i;

        if(i == j)
            ErrorFound("ER_01");
        else
        {
            p = i;
            f = 0;
            f |= ((TheLine[i] >= 'A' && TheLine[i] <= 'F') || (TheLine[i] == 'H') || (TheLine[i] == 'L') || (TheLine[i] == 'M')) ? 0 : 1;
            if(f)
                ErrorFound("ER_03");
            else
            {
                Param = TheLine[p];
                ChangedRegs.push_back(Param);
                i++;
                while(TheLine[i] == ' ')
                    ++i;
                if(TheLine[i] != ',')
                    ErrorFound("ER_01");
                else
                {
                    i++;
                    while(TheLine[i] == ' ')
                        ++i;
                    if(i == j)
                        ErrorFound("ER_01");
                    else if(TheLine[i] == '0')
                    {
                        if(TheLine[i+1] == 'X')
                        {
                            if(j > (i+6))
                                ErrorFound("ER_06");
                            else if(j == (i+2))
                                ErrorFound("ER_06");
                            else if(j == (i+3))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                if(f)
                                    ErrorFound("ER_06");
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    if(!SimCall(Command, Param, x))
                                        ErrorFound("ER_05");
                                    else
                                    {
                                        Command += " ";
                                        Command += Param;
                                        Memory[MemLoc] = this->InstCode[Command];
                                        MemInstr[MemLoc] = Command;
                                        MemLoc++;
                                    }
                                }
                            }
                            else if(j == (i+4))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                if(f)
                                    ErrorFound("ER_06");
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!SimCall(Command, Param, x))
                                        ErrorFound("ER_05");
                                    else
                                    {
                                        Command += " ";
                                        Command += Param;
                                        Memory[MemLoc] = this->InstCode[Command];
                                        MemInstr[MemLoc] = Command;
                                        MemLoc++;
                                    }
                                }
                            }
                            else if(j == (i+5))
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                                if(f)
                                    ErrorFound("ER_06");
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!SimCall(Command, Param, x))
                                        ErrorFound("ER_05");
                                    else
                                    {
                                        Command += " ";
                                        Command += Param;
                                        Memory[MemLoc] = this->InstCode[Command];
                                        MemInstr[MemLoc] = Command;
                                        MemLoc++;
                                    }
                                }
                            }
                            else
                            {
                                f = 0;
                                f |= ((TheLine[i+2] >= '0' && TheLine[i+2] <= '9') || (TheLine[i+2] >= 'A' && TheLine[i+2] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+3] >= '0' && TheLine[i+3] <= '9') || (TheLine[i+3] >= 'A' && TheLine[i+3] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+4] >= '0' && TheLine[i+4] <= '9') || (TheLine[i+4] >= 'A' && TheLine[i+4] <= 'F')) ? 0 : 1;
                                f |= ((TheLine[i+5] >= '0' && TheLine[i+5] <= '9') || (TheLine[i+5] >= 'A' && TheLine[i+5] <= 'F')) ? 0 : 1;
                                if(f)
                                    ErrorFound("ER_06");
                                else
                                {
                                    sscanf(TheLine+i+2, "%X", &x);
                                    Param = TheLine[p];
                                    if(!SimCall(Command, Param, x))
                                        ErrorFound("ER_05");
                                    else
                                    {
                                        Command += " ";
                                        Command += Param;
                                        Memory[MemLoc] = this->InstCode[Command];
                                        MemInstr[MemLoc] = Command;
                                        MemLoc++;
                                    }
                                }
                            }
                        }
                        else
                            ErrorFound("ER_06");
                    }
                }
            }
        }
    }
}

void MainWin::GenerateOpCode()
{
    char ActiveLine[100];

    CurrentLine = 0;
    InputFile.open(ui->lblFileName->text().toUtf8());

    while(!InputFile.eof())
    {
        ++ CurrentLine;
        InputFile.getline(ActiveLine,100);

        if(strlen(ActiveLine) < 1)
            continue;
        else if(ActiveLine[0] == ';')
            continue;

        CAPITALIZE(ActiveLine);
        GenOpCodeForLine(ActiveLine);
    }

    InputFile.close();
}

void MainWin::on_btnCompile_clicked()
{
    if(!on_action_Save_triggered())
        return;

    QString K;
    OldError = 0;
    ErrorCounter = 0;
    ALL_ERRORS.clear();
    ALL_ERROR_LINES.clear();

    ui->txtMsgs->setPlainText(QString("COMPILING ") + ui->lblFileName->text() + QString(" ..."));

    this->GenerateOpCode();

    on_actionRefresh_triggered();

    for(unsigned k = 0; k < ALL_ERROR_LINES.size(); ++k)
        ui->txtMsgs->setPlainText(ui->txtMsgs->toPlainText() + QString("\n") + K.setNum(ALL_ERROR_LINES[k]) + QString(" -- ") + K.fromStdString(ErrorMsgs[ALL_ERRORS[k]]));

    ui->txtMsgs->setPlainText(ui->txtMsgs->toPlainText() + QString("\n... COMPILED WITH ") + K.setNum(ErrorCounter) + QString(" ERROR(S)."));

    InputFile.open(ui->lblFileName->text().toUtf8());

    ui->btnCompile->setEnabled(false);
    ui->btnNext->setEnabled(true);

    MemLoc = 0;
    CurrentLine = 0;
}

void MainWin::on_btnNext_clicked()
{
    char ActiveLine[20];

    if(!InputFile.eof())
    {
        ++CurrentLine;

        InputFile.getline(ActiveLine,100);
        if(strlen(ActiveLine) < 1)
            return;

        ChangedRegs.clear();
        for(int k = 0; k < 7; ++k)
            ui->tableReg->item(k, 0)->setBackgroundColor(Qt::white);
        ChangedFlags.clear();
        for(int k = 0; k < 5; ++k)
            ui->tableFlag->item(k, 0)->setBackgroundColor(Qt::white);

        AnalyzeALine(ActiveLine);

        for(unsigned int k = 0; k < ChangedRegs.size(); ++k)
            ui->tableReg->item(IndexOfRegister[ChangedRegs[k]], 0)->setBackground(Qt::yellow);
        for(unsigned int k = 0; k < ChangedFlags.size(); ++k)
            ui->tableFlag->item(IndexOfFlag[ChangedFlags[k]], 0)->setBackgroundColor(Qt::green);

        on_actionRefresh_triggered();
    }
    else
    {
        ui->btnCompile->setEnabled(true);
        ui->btnNext->setEnabled(false);
        InputFile.close();
    }
}

void MainWin::on_btnSimulate_clicked()
{
    ui->btnSimulate->setEnabled(false);
    if(ui->btnCompile->isEnabled())
        this->on_btnCompile_clicked();
    ui->btnNext->setHidden(true);

    Sim.start(ui->sldDelay->value());
}

void MainWin::ExecuteStep()
{
    if(ui->btnNext->isEnabled())
    {
        this->on_btnNext_clicked();
        Sim.start(ui->sldDelay->value());
    }
    else
    {
        ui->btnSimulate->setEnabled(true);
        ui->btnNext->setHidden(false);
        ui->btnCompile->setEnabled(true);
        Sim.stop();
    }
}

void MainWin::on_sldDelay_valueChanged(int value)
{
    QString X;
    ui->lblDelay->setText(X.setNum(value));
}

void MainWin::on_actionAbout_Qt_triggered()
{
    QMessageBox::aboutQt(this);
}
