#ifndef MAINWIN_H
#define MAINWIN_H

#include <QFile>
#include <QTime>
#include <QLabel>
#include <QTimer>
#include <about.h>
#include <Headers.h>
#include <QFileDialog>
#include <QMainWindow>
#include <QMessageBox>
#include <QTableWidget>
#include <QSplashScreen>
#include <QDesktopWidget>

using namespace std;

namespace Ui
{
    class MainWin;
};

class MainWin : public QMainWindow
{
    Q_OBJECT

public:
    int Stack[65536];                                   // The Stack
    int Memory[65536];                                  // The Memory
    string MemInstr[65536];
    map<string, int> IndexOfRegister;         // Index of all registers

    void Initialize();
    explicit MainWin(QWidget *parent = 0);
    ~MainWin();

private:
    QTimer Sim;

    vector<string> Instruction;               // List of all commands
    vector<string> ChangedRegs;
    vector<string> ChangedFlags;

    int Flag[5];                                        // List of all flags
    int OFlag[5];                                       // List of all flags
    int Port[256];                                      // List of all ports
    int Register[13];                                   // List of all registers
    string OpCode[256];

    map<string, int> Limit;
    map<string, int> InstCode;
    map<string, int> Parameters;              // Parameters for each instruction
    map<string, int> IndexOfFlag;             // Index of all flags
    map<string, string> ErrorMsgs;       // OpCodes of instructions

    vector<string> ALL_ERRORS;
    vector<int> ALL_ERROR_LINES;

    QTableWidgetItem MemItems[20][2];
    QTableWidgetItem RegItems[7];
    QTableWidgetItem FlaItems[5];
    QTableWidgetItem StaItems[7][2];
    Ui::MainWin *ui;
    about *MyAbout;
    QString CodeStatus;
    bool FirstSave;
    QString FileName;
    QApplication *MyApp;
    void GenerateOpCode();
    void AnalyzeALine(char* TheLine);
    void GenOpCodeForLine(char* TheLine);

    //8085_FUNCTIONS
    void CP(int a);
    void RRC();
    void RLC();
    void JMP(int a);
    void PCHL();
    void STC();
    void RC();
    void RP();
    void RM();
    void RPE();
    void RPO();
    void ANI(int n);
    void PUSH(string a);
    void RAL();
    void RAR();
    void SBI(int n);
    void SBB(string r);
    void SUI(int n);
    void SUB(string r);
    void DCR(string r);
    void DAD(string r);
    void LHLD (int a);
    void LDAX (string a);
    void LDA (int a);
    void JPO (int address);
    void JC (int address);
    void JNZ (int address);
    void JNC (int address);
    void JPE (int address);
    void JP (int address);
    void XRI (int address);
    void XRA (string a);
    void DCX (string a);
    void XCHG();
    void CZ(int a);
    void CPI(int a);
    void CPO(int a);
    void CNZ(int a);
    void CPE(int a);
    void CNC(int a);
    void NOP();
    void JM(int address);
    void LXI (string a, int nn);
    void RZ ();
    void ORA(string a);
    void ORI(int n);
    void CMC();
    void CMA();
    void CMP(string a);
    void ACI(int n);
    void ADC(string a);
    void ADD(string a);
    void ADI(int n);
    void ANA(string a);
    void CALL(int address);
    void RNZ();
    void RNC();
    void RET();
    void CM(int address);
    void CC(int address);
    void MOV(string a, string b);
    void MVI (string a, int n);
    void XTHL();
    void POP(string register_name);

    bool Call(string a, string x, int p);
    bool Call(string a, string x, string y);
    bool Call(string a, string x);
    bool Call(string a, int p);
    bool Call(string a);

    bool SimCall(string a, string x, int p);
    bool SimCall(string a, string x, string y);
    bool SimCall(string a, string x);
    bool SimCall(string a, int p);
    bool SimCall(string a);

    //8085 Initialization
    void Init_ASM();
    void Init_OpCodes();
    void Init_Values();
    void Init_Errors();
    void Init_Indices();

    //8085 System Management
    void UpdateF();
    void UpdateAF();
    void UpdateBC();
    void UpdateDE();
    void UpdateHL();
    void UpdateB_C();
    void UpdateD_E();
    void UpdateH_L();
    void UpdateA_F();
    void UpdateFromF();
    void SetParity(int X);
    void UpdateFlags(int Address);
    void ErrorFound(string ErrorCode);
    void NormalizeMemory(int Address);
    void UpdateFlags(string Result);
    void NormalizeRegister(string Reg);

private slots:
    void on_actionAbout_Qt_triggered();
    void on_sldDelay_valueChanged(int value);
    void ExecuteStep();
    void on_btnSimulate_clicked();
    void on_btnNext_clicked();
    void on_btnCompile_clicked();
    void on_sldMem_valueChanged(int value);
    void on_sldMem_sliderReleased();
    void on_action_New_triggered();
    void on_actionRefresh_triggered();
    bool on_actionSave_As_triggered();
    void on_txtSource_textChanged();
    void on_action_Paste_triggered();
    void on_actionC_opy_triggered();
    void on_action_Cut_triggered();
    void on_action_Select_All_triggered();
    void on_action_Redo_triggered();
    void on_txtSource_undoAvailable(bool b);
    void on_txtSource_redoAvailable(bool b);
    void on_action_Undo_triggered();
    bool on_action_Save_triggered();
    void on_action_Open_triggered();
    void on_actionExit_triggered();
    void on_action_About_triggered();
};

#endif // MAINWIN_H
